#!/usr/bin/perl
#
# Verschiedene Funktionen zur automatischen Hardware-Erkennung
# 11.10.2003 by <cooper at linvdr.org>
# 
# Erweitert zum Setup-Programm
# 11.10.2003 by <tom at linvdr.org>
# 

use strict;
use Getopt::Std;
use POSIX qw(strftime sprintf setlocale);
use UI::Dialog::Backend::CDialog;
use LWP::UserAgent;
use Locale::gettext;
use File::Basename;

sub true() { 1; }
sub false() { 0; }

setlocale(LC_MESSAGES, GetLanguage());
textdomain("SETUP");

my $VERSION      = "0.7";
my $KERNEL       = `uname -r`;
chomp($KERNEL);
my $SYSCONFIG    = "/etc/sysconfig";
my $GRUB_CONFIG  = "/boot/grub/grub.conf";
my $TITLE        = "LinVDR - Setup";
my $PLUGINDIR    = "/usr/lib/vdr/plugins";
my $NETWORKDIR   = "/lib/modules/$KERNEL/kernel/drivers/net";
my $LANG         = "de";
my $DEVICE_CDROM = "/dev/cdrom";
my $PCIMAP       = "/lib/modules/$KERNEL/modules.pcimap";
my $PCIDB        = "/etc/pci.db"; # http://pciids.sourceforge.net/pci.db
my $SMBCONF      = "/etc/samba/smb.conf";
my $DISCLAIMER   = "/" . gettext("README.de.install");
my $CHANNELSCONF = "/etc/vdr/channels.conf";
my $MODULES_SETUP= "/etc/modules.setup";
my %CONFIG;

my $PLUGINS = {
	"ac3mode"		=> gettext("Zeigt die Anzahl der Audio-Kanle"),
	"actuato"		=> gettext("Ansteuerung eines Sat-Schssel-Rotors"),
	"admin"			=> gettext("Administrative Aufgaben"),
	"advchctrl"		=> gettext("Audiopegel pro Kanal festlegen"),
	"adzap"			=> gettext("Werbezapper"),
	"aide"			=> gettext("OSD-Hilfesystem"),
	"alcd"			=> gettext("Ausgabe von Infomationen auf Activy300-LC-Displays"),
	"amarok"		=> gettext("amaroK frontend"),
	"analogradio"		=> gettext("analoge Radio-Karte als Eingabegert"),
	"analogtv"		=> gettext("analoge TV-Karte als MPEG-Eingabegert"),
	"archive"		=> gettext("Multimedia-Archiv zur Verwaltung fr z.B DVD oder VHS via OSD"),
	"arghdirector"		=> gettext("Erweiterung des director-plugins"),
	"asterisk"		=> gettext("Client fr eine Asterisk-Sprachbox"),
	"atmolight"		=> gettext("Atmolight Plugin (Nachbau des Philips Ambilight)"),
	"audiorecoder"		=> gettext("unbekannt"),
	"autosort"		=> gettext("Sortieren der Kanle"),
	"autotimer"		=> gettext("Automatisches Anlegen von Timern"),
	"autotimeredit"		=> gettext("OSD-Autotimer fr vdradmin"),
	"avolctl"		=> gettext("Lautstrkeregler der Audiokarte steuern"),
	"beep"			=> gettext("Ausgabe akustischer Meldungen ber den PC Lautsprecher"),
	"bitstreamout"		=> gettext("Dolby-Digital-Ausgabe ber eine Audiokarte (ber Alsa)"),
	"btrcu"			=> gettext("Bluetooth-Mobiltelefon als Fernbedienung"),
	"burn"			=> gettext("Brennt mehrere Filme auf eine DVD"),
	"calc"			=> gettext("Kleiner Taschenrechner"),
	"calendar"		=> gettext("Zeigt EPG-Daten in Form eines Kalenders"),
	"cc"			=> gettext("Zeigt verborgene Untertitel"),
	"ccd400"		=> gettext("Steuerung einer CCD400-Kamera"),
	"cdda"			=> gettext("Audio-CD-Spieler"),
	"channelscan"		=> gettext("Scannt Satellitentransponder und aktualisiert die Kanle"),
	"channelswitcher"	=> gettext("Umschalten zwischen mehreren Kanaleinstellungen"),
	"chanorg"		=> gettext("Sortiert Kanle"),
	"cinebars"		=> gettext("Erzeugt virtuelle Kinostreifen"),
	"clock"			=> gettext("Einblendung der Uhrzeit"),
	"console"		=> gettext("Textkonsole auf dem Fernseher"),
	"control"		=> gettext("Ausgabe des OSD auf einem Telnet-Terminal"),
	"csf"			=> gettext("Sortiert Kanle"),
	"csmon"			=> gettext("Ein Cardserver-Monitor"),
	"daap"			=> gettext("Ein Digital Audio Access Protocol'-Client"),
	"decruft"		=> gettext("Lscht Kanle nach Vorgaben"),
	"digicam"		=> gettext("Zugriff auf Digitalkameras"),
	"director"		=> gettext("Multifeedkanle von Premiere nutzen"),
	"dsmcc"			=> gettext("Mhp-Datenkarussell (alpha)"),
	"dummydevice"		=> gettext("Dummy-Ausgabegert"),
	"dv"			=> gettext("Zugriff auf Camcorder"),
	"dvd"			=> gettext("DVD-Spieler"),
	"dvdconvert"		=> gettext("dvd2vdr"),
	"dvdrepg"		=> gettext("verteilt die Daten des EPG im Netz (VIDEGOR)"),
	"dvdrsched"		=> gettext("ordnet Aufnahmeauftrge den einzelnen Videorekordern zu (VIDEGOR)"),
	"dvdrvidtrans"		=> gettext("bertrgt Aufzeichnungen zwischen den Videorekordern (VIDEGOR)"),
	"dvdselect"		=> gettext("DVD-Laufwerke einhngen"),
	"dvdswitch"		=> gettext("Abspielen von DVD-Images"),
	"dxr3"			=> gettext("DXR 3/Hollywood+-Karte als MPEG-Ausgabegert"),
	"em84xx"		=> gettext("em84xx als MPEG-Ausgabegert"),
	"eggtimer"		=> gettext("Eieruhr fr den Fernsehschtigen"),
	"epgsearch"		=> gettext("EPG durchsuchen, Suchtimer zum automatischen anlegen von Timern, Prfen auf Timerkonflikte"),
	"epgsearchonly"		=> gettext("Separater Menuepunkt fr das EPG-Search Plugin"),
	"extb"			=> gettext("Steuerung des Extension-Boards"),
	"externalplayer"	=> gettext("externe Programme als Player"),
	"extrecmenu"		=> gettext("erweitertes Aufzeichnungs-Men"),
	"femon"			=> gettext("Anzeige der DVB-Signalqualitt"),
	"fepg"			=> gettext("Zeigt EPG grafisch + Navigation"),
	"ffnetdev"		=> gettext("Full-Featured-DVB-Device-Emulation bers Netzwerk"),
	"filebrowser"		=> gettext("Dateimanager"),
	"freecell"		=> gettext("Kartenspiel Freecell"),
	"fussball"		=> gettext("Zeigt die 1.Bundesligatabelle und Spielergebnisse an"),
	"soccer"		=> gettext("Zeigt die 1.Bundesligatabelle und Spielergebnisse an"),
	"games"			=> gettext("Eine kleine Spielesammlung"),
	"gbc"			=> gettext("GameBoy-Emulator"),
	"gngb2vdr"		=> gettext("GameBoy-Emulator"),
	"graphlcd"		=> gettext("Ausgabe von Informationen auf grafischen LCDs"),
	"graphtft"		=> gettext("Ausgabe von Informationen auf TFTs"),
	"gstreamer"		=> gettext("Experimentell, uneindeutiger Zweck"),
	"iaxphone"		=> gettext("Ein einfaches 'Softphone'"),
	"image"			=> gettext("Bildbetrachter"),
	"instantimer"		=> gettext("Einfaches Einstellen fr einen (SW-)Timer fr das kommende Event"),
	"ipod"			=> gettext("Zeigt ITunesDB des iPod an"),
	"isdnlog"		=> gettext("ISDN-Log"),
	"joystick"		=> gettext("Verwendung eines Joysticks als Fernbedienung"),
	"kathreinlcd"		=> gettext("IC-Displays"),
	"kvdrmon"		=> gettext("Hilfsplugin fr kvdrmon, einem KDE Kicker-Applet zur berwachung des VDR-Status "),
	"launcher"		=> gettext("startet andere Plugins"),
	"lcdproc"		=> gettext("Ausgabe von Informationen auf LCDs"),
	"lcr"			=> gettext("Least-Cost-Routing-Plugin"),
	"lirc"			=> gettext("Testen von LIRC"),
	"lircrc"		=> gettext("Native LIRC-Untersttzung"),
	"loadepg"		=> gettext("Import von EPG-Daten (Canal+ group)"),
	"locker"		=> gettext("Kinderschutz - Sendungen sperren"),
	"logowipeout"		=> gettext("Sender Logo-Killer"),
	"logviewer"		=> gettext("VDR/Systemlogs live anschauen"),
	"mailbox"		=> gettext("Einfacher eMail-Client"),
	"manual"		=> gettext("Einlesen von XML-Hilfedateien"),
	"mcast"			=> gettext("Machbarkeitsstudie Multicast im Netzwerk"),
	"mediamvp"		=> gettext("Hauppauge Media-MVP"),
	"mvp"			=> gettext("Hauppauge Media-MVP"),
	"message"		=> gettext("Schnittstelle fr Shell-Skripte"),
	"mhp"			=> gettext("Mhp fr VDR"),
	"mlcd"			=> gettext("Ausgabe von Infomationen auf Siemens Multitainer-Displays"),
	"mldkgui"		=> gettext("MLDonkey-Plugin"),
	"mldonkey"		=> gettext("MLDonkey-Plugin"),
	"mlist"			=> gettext("Zeigt eine Liste aller OSD-Nachrichten an"),
	"monochannel"		=> gettext("Experimentell, Audio L/R Umschaltung"),
	"mosaic"		=> gettext("Blttern im EPG der mosaic Kanle von canalsat"),
	"mp3"			=> gettext("MP3-Spieler"),
	"mp3ng"			=> gettext("Optisch ein wenig erweiterter MP3-Spieler"),
	"mpcsmon"		=> gettext("unbekannt"),
	"mplayer"		=> gettext("Spielt verschiedene Videoformate ab (z.B. DivX)"),
	"mplayercluster"	=> gettext("MPlayer im Verbund"),
	"muggle"		=> gettext("MP3-Spieler mit Datenbank"),
	"netconfig"		=> gettext("Netzwerkeinstellungen + Freigaben (smb) ndern"),
	"network"		=> gettext("Netzwerkeinstellungen ndern"),
	"newsticker"		=> gettext("Nachrichten in Laufschrift am unteren Bildrand einblenden"),
	"noepgmenu"		=> gettext("Umsetzung des noepg-patch als Plugin"),
	"nordlichtsepg"		=> gettext("Erweiterte EPG-Anzeige"),
	"openmhp"		=> gettext("freie MHP-Implementierung"),
	"osdimage"		=> gettext("Ausgabe von Bildern ber das OSD"),
	"osdout"		=> gettext("Experimentell, Ausgabe des OSD ber FB"),
	"osdpip"		=> gettext("Bild-im-Bild"),
	"osdteletext"		=> gettext("Teletext-Dekoder fr das OSD"),
	"otv4vdr"		=> gettext("TPS EPG-Import"),
	"pcd"			=> gettext("Spielt Kodak-Photo-CDs ab"),
	"perl"			=> gettext("Experimentell, Perl extension VDR"),
	"pilot"			=> gettext("Zapping Co-Pilot"),
	"pilotskin"		=> gettext("Blttern im EPG ohne Kanalumschalten"),
	"pim"			=> gettext("Personal information manager"),
	"pin"			=> gettext("Kindersicherung fr Aufnahmen, Plugins und Einstellungen"),
	"playlist"		=> gettext("Verwendung von Playlisten fr Videoaufnahmen"),
	"pluginsetup"		=> gettext("Managen der Plugins ber das OSD"),
	"podcatcher"		=> gettext("Ldt und spielt podcasts ab"),
	"powermate"		=> gettext("Verwendung des Griffin PowerMate zur Steuerung einiger Funktionen"),
	"prefermenu"		=> gettext("Bevorzugte Kanle"),
	"premiereepg"		=> gettext("premiereepg2vdr als Plugin"),
	"proxy"			=> gettext("Ldt andere Plugins"),
	"ptv"			=> gettext("Hauppauge PVR x50 als MPEG-Eingabegert"),
	"pvr350"		=> gettext("Hauppauge PVR 350 als MPEG-Ausgabegert"),
	"pvrinput"		=> gettext("Hauppauge PVR x50 als MPEG-Eingabegert"),
	"pvrusb2"		=> gettext("Hauppauge PVR USB2 als MPEG-Eingabegert"),
	"quicktimer"		=> gettext("Schnelles Anlegen von Timern durch Angabe von Kanal, Tag, Zeit"),
	"radio"			=> gettext("RDS und Hintergrundbild fr Radiosender"),
	"radioinfo"		=> gettext("Zusatzinfos fr Radiosender darstellen"),
	"radiolist"		=> gettext("Kanalliste fr Radiosender"),
	"recstatus"		=> gettext("Aufnahmestatus der Aufnahmegerte"),
	"reelchannelscan"	=> gettext("Scannt Satellitentransponder und erzeugt/aktualisiert die channels.conf"),
	"relay"			=> gettext("Ansteuerung eines Relais ber serielle Schnittstelle"),
	"remote"		=> gettext("Untersttzung zustzlicher Fernbedienungen (FB der Hauppauge Nexus)"),
	"remoteosd"		=> gettext("Zeigt das Men eines entfernten VDR als Untermen im lokalen VDR an"),
	"rockantenne"		=> gettext("Zeigt Titel des aktuellen Songs auf Radio Rockantenne"),
	"rotor"			=> gettext("Ansteuerung eines Sat-Schsselrotors"),
	"rssreader"		=> gettext("Stellt RSS-Informationen dar"),
	"safe"			=> gettext("Kommandos mit Codes versehen"),
	"savechannel"		=> gettext("Speichern des Kanals bei jedem Umschalten"),
	"sc"			=> gettext("Software-CAM (Conditional Access Modul)"),
	"screenshot"		=> gettext("Erstellt Screenshots"),
	"serial"		=> gettext("Steuerung des VDR ber Tasten an der Seriellen Schnittstelle"),
	"serialremote"		=> gettext("Experimentell, fr UIRT IR Empfnger"),
	"serled"		=> gettext("Ansteuerung von 2 Status-LEDs ber die Serielle Schnittstelle"),
	"setup"			=> gettext("Plugin zum Bearbeiten von Systemeinstellungen / VDR-Mens usw. per OSD"),
	"setuploadandsave"	=> gettext("Demo, setup.conf bei laufendem VDR einlesen und speichern"),
	"showrecinfo"		=> gettext("Detailinfos zu vorhandenen Aufnahmen"),
	"sked"			=> gettext("Softcam.Key EDitor"),
	"skinclassic"		=> gettext("Umsetzung des klassischen vdr-skins als Plugin"),
	"skincurses"		=> gettext("Skin fr die Konsole"),
	"skinelchi"		=> gettext("Elchi-Patch als Plugin"),
	"skinsoppalusikka"	=> gettext("Umsetzung des Elchi-Patch als Plugin"),
	"skinnotrans"		=> gettext("Transparente Menus fr das softdevice-plugin"),
	"skinreel"		=> gettext("Standard-Skin der reelbox"),
	"sky"			=> gettext("mpeg2-Encoder-Plugin (auch als 'dummy'-Input)"),
	"sleeptimer"		=> gettext("Ausfhren von Befehlen/Shutdown mit 'countdown'-Zhler"),
	"snapshot"		=> gettext("Erstellt Screenshots"),
	"softdevice"		=> gettext("X-Server (XV), Framebuffer oder Vidix als Ausgabegert"),
	"softdevice2net"	=> gettext("Netzwerk-Ausgabegert statt Framebuffer"),
	"softieee1394"		=> gettext("Steuern von Firewire Camcordern"),
	"softmpeg"		=> gettext("Software MPEG decoder"),
	"softplay"		=> gettext("Spielt Mediadateien per softdevice-plugin ab"),
	"solitaire"		=> gettext("Kartenspiel Solitr"),
	"soundtest"		=> gettext("Visualisierung der Soundkarte ber das OSD"),
	"spider"		=> gettext("Kleines OSD-Kartenspiel"),
	"statusleds"		=> gettext("Nutzt die LEDs der Tastatur zur Anzeige laufenden Aufnahmen"),
	"statusandquestion"	=> gettext("Erweiterung des SVDRP"),
	"streamdev-client"	=> gettext("Streaming-Client"),
	"streamdev-server"	=> gettext("Streaming-Server"),
	"streamplayer"		=> gettext("Spielt Videostreams ab"),
	"submenu"		=> gettext("Erstellen von Untergruppen im Men"),
	"subtitles"		=> gettext("Anzeige von DVB-Untertitel"),
	"sudoku"		=> gettext("Erzeugen und Lsen von Zahlenpuzzles"),
	"surfer"		=> gettext("News-Browser"),
	"suspendoutput"		=> gettext("Hintergrundbild fr Radiosender"),
	"svdrpext"		=> gettext("Implementation zustzlicher SVDRP Kommandos"),
	"svdrppara"		=> gettext("Abfrage und Setzen einiger Parameter des VDRs per SVDRP"),
	"svdrpservice"		=> gettext("This plugin offers SVDRP connections as a service to other plugins"),
	"swcon"			=> gettext("Wechselt die Konsole"),
	"switchosdpos"		=> gettext("Manuelles Umschalten der OSD Position und Grsse"),
	"sysconfig"		=> gettext("VDR-Frontend zur Systemkonfiguration"),
	"sysinfo"		=> gettext("Systeminformationen via OSD"),
	"taste"			=> gettext("Sendungen anhand von Keywords sperren"),
	"text2skin"		=> gettext("Ein Lader fr OSD-Skins"),
	"timeline"		=> gettext("Zeigt Kollisionen programmierter Timer"),
	"transfron"		=> gettext("Oberflche fr Transcode/MPeg4ip"),
	"trayopen"		=> gettext("ffnet das CD/DVD-Laufwerk"),
	"ttxtsubs"		=> gettext("Teletext-Untertitel"),
	"tvonscreen"		=> gettext("EPG-Viewer im Fernsehzeitungsformat"),
	"tvtv"			=> gettext("TvTv im OSD"),
	"txtsubs"		=> gettext("Liet Untertitel"),
	"umsadmin"		=> gettext("Oberflche und Erweiterungen fr Capisuite"),
	"undelete"		=> gettext("Wiederherstellung gelschter Aufnahmen"),
	"usbremote"		=> gettext("Remote-Plugin fr Plug-USB (AVR)"),
	"vbox"			=> gettext("Oberflche fr den VBox-Anrufbeantworter"),
	"vcd"			=> gettext("S/VCD-Spieler"),
	"vdrcd"			=> gettext("AutoPlay-Funktion"),
	"vdrconvert"		=> gettext("Erzeugen und Brennen von DVDs"),
	"vdricq"		=> gettext("ein ICQ-Client-Plugin"),
	"vdrrip"		=> gettext("Film-Encoder"),
	"videosystem"		=> gettext("OSD-Einstellungen abhngig vom Video-System ndern (PAL/NTSC)"),
	"viewer"		=> gettext("Anzeigen von Systemlogs"),
	"vod"			=> gettext("Video on Demand"),
	"vompserver"		=> gettext("Ansteuerung einer Hauppauge MediaMVP"),
	"wapd"			=> gettext("Fernsteuerung durch WAP"),
	"weatherng"		=> gettext("Wetterinformationen via WWW"),
	"webepg"		=> gettext("EPG-Ersatz"),
	"x10"			=> gettext("steuern von X10-Devices"),
	"x11out"		=> gettext("X11 auf der DVB-Karte ausgeben"),
	"xine"			=> gettext("Software-Emulation eines VDR-Ausgabegertes"),
	"xineliboutput"		=> gettext("X11-Frontend fr VDR"),
	"xms"			=> gettext("Medienverwaltung"),
	"xxvautotimer"		=> gettext("Autotimer fr xxv ber das OSD bearbeiten"),
	"yaepg"			=> gettext("Elektronischer Programmfhrer"),
	"zaphistory"		=> gettext("Liste der zuletzt geschauten Programme mit Statistik"),
#Demo-Plugins
	"ditherndemo"		=> gettext("Demo-Plugin"), 	
	"epgsearchclient"	=> gettext("Demo-Plugin"), 	
	"femonclient"		=> gettext("Demo-Plugin"), 	
	"hello"			=> gettext("Demo-Plugin: Ein einfaches 'hello'-Plugin"),
	"osddemo"		=> gettext("Demo-Plugin: On-Screen-Display"),
	"osdtest256"		=> gettext("Demo-Plugin: Farbbalkentest fr DVB Karten mit 4MB SDRAM"),
	"parallelportcontroller"=> gettext("Demo-Plugin: Zeigt Hardwareansteuerung und OSD-Malerei"),
	"rtextdemo"		=> gettext("Demo-Plugin"), 	
	"servicedemo"		=> gettext("Demo-Plugin zur Kommunikation zwischen Plugins"),
	"statusbits"		=> gettext("Demo-Plugin"), 	
	"status"		=> gettext("Demo-Plugin: OSD-Status"),
	"svdrpdemo"		=> gettext("Demo-Plugin"), 	
	"svcsvr"		=> gettext("Demo-Plugin zur Kommunikation zwischen Plugins"), 	
	"svccli"		=> gettext("Demo-Plugin zur Kommunikation zwischen Plugins"), 	
#Nicht mehr gepflegte Plugins (sollten auch nicht mehr benutzt werden)
	"audiocd"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"browse"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"), 	
	"commscan"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"led"			=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"mp3oss"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"stream"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"streamc"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"browser"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"tetris"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"tictactoe"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"telecable"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"), 	
	"teletext"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"vdrc"			=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"vdrmail"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"vdrrec"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"), 	
	"weather"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
	"xvxine"		=> gettext("Dieses Plugin wird nicht mehr geflegt!"),
};

my $d = new UI::Dialog::Backend::CDialog ( 
  # XXX autosize dont work with checkbox
  backtitle => $TITLE,
  debug => 0, 
  height => 20, 
  width => 70,
  "ok-label" => gettext("Ok"),
  "cancel-label" => gettext("Abbruch"));
my($return, @return);
my $professional_mode = true;

my %args;

getopts("axh", \%args);
if($args{a}) {
  ReadConfig();
  Assistant();
} elsif($args{h}) {
  printf("Usage %s [OPTION]...\n\n", join("", $0 =~ /^.*\/(.*)/));
  printf("  -a  run assistant\n");
  printf("  -h  help\n");
  printf("Report bugs to <linvdr\@linvdr.org>\n");
  exit(0);
} elsif($args{x}) {
  $professional_mode = true;
  ReadConfig();
  Menu();
} else {
  if(ReadConfig()) {
    Assistant();
  } else {
    Menu();
  }
}

#############################################################################
# 
#
sub Assistant {
  #UmlautWarning();
  $d->textbox(
    title      => "",
    path       => $DISCLAIMER,
  );
  PluginsMenu();
  VDRSelectChannelsMenu(); 
  IRMenu();
  VDRFBTVMenu();
  NetworkLanNetworking();
  if($CONFIG{NETWORKING_LAN} eq "yes") {
    NetworkLanAutodetectMenu();
    NetworkLanDHCPMenu();
    if($CONFIG{USE_DHCP_LAN} eq "no") {
      NetworkLanIPMenu();
      NetworkLanGatewayMenu();
      NetworkLanDomainMenu();
      NetworkLanDNSMenu();
    }
    FileSharingStartSamba();
  }
  DetectCDROMMenu();
  $d->msgbox(
    text       => [gettext("Setup erfolgreich abgeschlossen.\n\nDas Setup kann jederzeit wieder mit dem Kommando \"setup\" gestartet werden.")],
    width      => 50,
    height     => 10,
    "ok-label" => gettext("Beenden")
  );
  WriteConfig();
}

sub UmlautWarning {
  if(readlink("/proc/self/fd/0") eq "/dev/tty1") {
    if(GetLanguage() =~ /^de_DE/) {
      $d->msgbox(
        text       => [gettext("Umlaut Problem im Kernel, Umlaute werden auf dieser Konsole nicht korrekt dargestellt.")],
        width      => 50,
        height     => 10,
        "ok-label" => gettext("Weiter")
      );
    }
  }
}

sub notRootWarning {
  if($< != 0) {
    $d->msgbox(
      text       => [gettext("Setup muss als Benutzer root gestartet werden.")],
      width      => 50,
      height     => 6,
      "ok-label" => gettext("Beenden")
    );
    exit(1);
  }
}

#############################################################################
# 
#
sub Menu {
  my $exit = 0;
  notRootWarning();
  #UmlautWarning(); # XXX
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("Hauptmen"),
      text  => gettext("Bitte auswhlen:"),
      menuheight => 9,
      height => 16,
      list  => [
        gettext("Netzwerk"), gettext("Netzwerk-Einstellungen"),
        gettext("Passwrter"), gettext("Passwrter"),
        gettext("VDR"),  gettext("VDR-Einstellungen"),
        gettext("Sharing"), gettext("Filesharing"),
        gettext("IR"), gettext("Infrarot-Empfnger"),
        gettext("CD/DVD"), gettext("CD-/DVD-ROM-Einstellungen"),
        gettext("Sonstiges"), gettext("Sonstiges"),
        ($professional_mode == true) ?
          (gettext("Erweitert"), gettext("'Du weit hoffentlich, was du tust!'-Modus")) : (),
        gettext("Beenden"),  gettext("Konfiguration verlassen"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("Netzwerk")) {
      NetworkLanMenu();
    } 
    if($return eq gettext("Passwrter")) {
      PasswordMenu();
    }
    if($return eq gettext("VDR")) {
      VDRMenu();
    } 
    if($return eq gettext("IR")) {
      IRMenu();
    } 
    if($return eq gettext("Sharing")) {
      FileSharingMenu();
    }
    if($return eq gettext("CD/DVD")) {
      CDROMMenu();
    }
    if($return eq gettext("Sonstiges")) {
      OtherMenu();
    }
    if($return eq gettext("Erweitert")) {
      AdvancedMenu();
    }
    if($return eq gettext("Beenden")) {
      $return = $d->yesno(
        title => gettext("Setup Beenden"),
        text  => [ gettext("Konfiguration speichern?\n\nDie meisten nderungen werden erst beim nchsten Start wirksam!") ],
        height=> 8,
      );
      if($return) {
        WriteConfig();
      }
      $exit = 1;
    }
  }
}

sub VDRMenu {
  my $exit = 0;
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("VDR Hauptmen"),
      text  => gettext("Bitte auswhlen:"),
      menuheight => 8,
      height => 15,
      list  => [
        gettext("Plugins"), gettext("VDR-Plugins auswhlen"),
        gettext("Kanle"), gettext("Kanalliste auswhlen"),
        gettext("FBTV"), gettext("TV-Bild auf dem Monitor darstellen"),
        gettext("Zurck"), gettext("Zurck in das vorherige Men"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("Plugins")) {
      PluginsMenu();
    } 
    if($return eq gettext("Kanle")) {
      VDRSelectChannelsMenu();
    }
    if($return eq gettext("FBTV")) {
      VDRFBTVMenu();
    } 
    if($return eq gettext("Zurck")) {
      $exit = 1;
    }
  }
}

sub VDRFBTVMenu {
  $return = $d->yesno(
    text => [gettext("Soll das Fernsehbild zustzlich auf dem Computermonitor angezeigt werden?\nHinweis: Der Fernseher von VDR-Admin funktioniert nur, wenn sie hier mit \"Nein\" antworten!")],
    defaultno => ($CONFIG{START_FBTV} eq "yes") ? 0 : 1,
    height => 8);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{START_FBTV} = "yes";
  } else {
    $CONFIG{START_FBTV} = "no";
  }
}

sub VDRSelectChannelsMenu {
  my @conf;
  for my $conf (GetChannelConfigs()) {
    my $selected;
    $selected = 1 if(basename(readlink($CHANNELSCONF)) eq $conf->{file});
    push(@conf, $conf->{file});
    push(@conf, [ $conf->{media} . " " . $conf->{host}, $selected ]);
  }

  $return = $d->radiolist(
    title => gettext("Kanalliste auswhlen"),
    list  => [ @conf ],
    menuheight => 12
  );
  Error() if($d->state ne "OK");
  if($return !~ /\.conf$/) {
    die "$return not expected!";
  }
  unlink($CHANNELSCONF) || die "cant remove $CHANNELSCONF";
  symlink("/etc/vdr/channels/$return", $CHANNELSCONF);
}

#############################################################################
# Passwords
#
sub PasswordMenu {
  my $exit = 0;
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("Passwrter ndern"),
      text  => gettext("Welche Passwrter sollen gendert werden?"),
      menuheight => 3,
      height => 10,
      list  => [
        gettext("System"), gettext("root-Zugang (ssh, scp, sftp)"),
        gettext("Samba"), gettext("Passwort fr Filesharing (Samba)"),
        gettext("Zurck"), gettext("Zurck in das vorherige Men"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("System")) {
      PasswordMenuSystemSetPassword();
    } 
    if($return eq gettext("Samba")) {
      FileSharingChangePassword();      
    }
    if($return eq gettext("Zurck")) {
      $exit = 1;
    }
  }
}

sub PasswordMenuSystemSetPassword {
  do {
    $return = $d->password(
      text => gettext("Neues Passwort fr den Benutzer root:"), 
      entry => "",
      height => 8);
    
    if($d->state() eq "OK") {
      if(length($return) >= 4) {
        my $retry = $d->password(
          text => gettext("Neues Passwort fr den Benutzer root wiederholen:"), 
          entry => "",
          height => 8);
        if($d->state eq "OK") {
          if($return eq $retry) {
            SystemSetPassword($return);
            $d->msgbox(
              height => 5,
              width => 30,
              #text => ["[A=CENTER]Password gendert."],
              text => [gettext("    Passwort gendert.")],
            );
            return;
          } else {
            $d->msgbox(
              text => gettext("Passwrter stimmen nicht berein!"),
              height => 5,
            );
            $return = "";
          }
        } else {
          Error();
        }
      } else {
        $d->msgbox(
          text => gettext("Das Passwort darf nicht krzer als vier Zeichen sein!"),
          height => 5,
        );
      }
    } else {
      Error();
    }
  } while(length($return) < 4);
}

#############################################################################
# Optical Drives
# 
sub CDROMMenu {
  my $exit = 0;
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("CD-/DVD-ROM-Einstellungen"),
      text  => "",
      menuheight => 3,
      height => 10,
      list  => [
        gettext("Search"), gettext("CD-/DVD-ROM automatisch erkennen"),
        gettext("Speed"), gettext("CD-/DVD-ROM-Geschwindigkeit setzen"),
        gettext("Zurck"), gettext("Zurck in das vorherige Men"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("Search")) {
      DetectCDROMMenu();
    } 
    if($return eq gettext("Speed")) {
      SetSpeedCDROMMenu();
    } 
    if($return eq gettext("Zurck")) {
      $exit = 1;
    }
  }
}

sub DetectCDROMMenu {
  my @cdrom = SearchCDROM();
  if(scalar(@cdrom) == 0) {
    $return = $d->msgbox(
      title => gettext("Suche nach CD-/DVD-ROM"),
      text => [gettext("Konnte kein CD-/DVD-ROM finden!\nDas Abspielen von DVDs/VCDs ist daher nicht mglich.")],
      height => 6
    );
    if($d->state() ne "OK") {
      Error();
    }
    return(1);
  } else {
    my $cdrom = "/dev/" . $cdrom[0];
    $d->msgbox(
      title => gettext("Suche nach CD-/DVD-ROM"),
      text => gettext("CD-/DVD-ROM an $cdrom gefunden."),
      height => 6,
      width => 45
    );
    # wir beruecksichtigen nur das erste CD/DVD-ROM
    unlink($DEVICE_CDROM);
    symlink($cdrom, $DEVICE_CDROM);
    $CONFIG{CDROM_DEVICE} = $cdrom;
    return(0);
  }
}

sub SetSpeedCDROMMenu {
  return if(DetectCDROMMenu());
  my @conf;
  for(my $count = 0; $count < 16; $count += 2) {
    my $selected;
    if(defined($CONFIG{CDROM_SPEED}) && $CONFIG{CDROM_SPEED} > 0) {
      if($CONFIG{CDROM_SPEED} == $count) {
        $selected = 1;
      }
    } else {
      if($count == 0) {
        $selected = 1;
      }
    }
    
    push(@conf, $count);
    
    my $text;
    if($count > 0) {
      $text = "${count}X";
    } else {
      $text = gettext("Keine nderung");
    }
    push(@conf, [ $text, $selected ]);
  }

  $return = $d->radiolist(
    title => gettext("CD-/DVD-ROM-Lesegeschwindigkeit einstellen"),
    text  => [ gettext("Die Lesegeschwindigkeit (1x entspricht 150 KB/s bei CDs und 1385 MB/s bei DVDs) beeinflusst mageblich das Betriebsgerusch des Laufwerks. Nicht alle CD-/DVD-ROMs untersttzen diese Funktion.") ],
    list  => [ @conf ],
    menuheight => 8,
    height=> 15,
  );
  return if($d->state ne "OK");
  if($return !~ /\d+/) {
    $CONFIG{CDROM_SPEED} = 0;
  } else {
    $CONFIG{CDROM_SPEED} = $return;
  }
}

#############################################################################
# Remote Control 
#
sub IRMenu {
  $return = $d->yesno(
    text => [gettext("Verwenden Sie die Original-Hauppauge-Fernbedienung und den Original-Hauppauge-Infrarot-Empfnger, direkt angeschlossen an der Original-Hauppauge-DVB-Karte?\n(Antworten Sie unbedingt mit \"Nein\", wenn Sie einen seriellen IR-Empfnger auf Basis des TSOP-1738 oder TSOP-1838 mit LIRC verwenden wollen!)")],
    defaultno => ($CONFIG{USE_INTERNAL_IR} eq "yes") ? 0 : 1,
    height => 10);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{USE_INTERNAL_IR} = "yes";
    #$return = $d->yesno(
    #  text => [gettext("Soll das Infrarot-Signal invertiert werden?\n(Fgt den Parameter \"-i\" beim Aufruf von \"av7110_loadkeys\" hinzu, ggf. bei Problemen ausprobieren.)")],
    #  defaultno => ($CONFIG{INVERT_SIGNAL} eq "yes") ? 0 : 1,
    #  height => 7);
    #if($d->state() eq "ESC") {
    #  Error();
    #}
    #if($return) {
    #  $CONFIG{INVERT_SIGNAL} = "yes";
    #} else {
    #  $CONFIG{INVERT_SIGNAL} = "no";
    #}
  } else {
    $CONFIG{USE_INTERNAL_IR} = "no";
  }
}

#############################################################################
# File Sharing
#
sub FileSharingMenu {
  my $exit = 0;
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("Filesharing-Einstellungen"),
      text  => "",
      menuheight => 8,
      height => 15,
      list  => [
        ($CONFIG{START_SAMBA} ne "yes") ?
          (gettext("Samba"), gettext("Samba einschalten")) :
          (gettext("Samba"), gettext("Samba ausschalten"),
          gettext("Arbeitsgruppe"), gettext("Arbeitsgruppe auswhlen"),
          gettext("Passwort"), gettext("Passwort ndern")),  
        ($CONFIG{START_FTPD} ne "yes") ?
          (gettext("ftpd"), gettext("FTP-Server einschalten")) :
          (gettext("ftpd"), gettext("FTP-Server ausschalten")),
        ($CONFIG{START_DAAPD} ne "yes") ?
          (gettext("daapd"), gettext("daapd einschalten (iTunes)")) :
          (gettext("daapd"), gettext("daapd ausschalten (iTunes)")),
        gettext("Zurck"), gettext("Zurck in das vorherige Men"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("Samba")) {
      FileSharingStartSamba();
    }
    if($return eq gettext("Arbeitsgruppe")) {
      FileSharingSelectWorkgroup();
    }
    if($return eq gettext("Passwort")) {
      FileSharingChangePassword();
    }
    if($return eq gettext("ftpd")) {
      FileSharingStartFTPD();
    }
    if($return eq gettext("daapd")) {
      FileSharingStartDAAPD();
    }
    if($return eq gettext("Zurck")) {
      $exit = 1;
    }
  }
}

sub FileSharingStartSamba {
  $return = $d->yesno(
    text => [gettext("Soll der Samba-Server gestartet werden? Damit knnen Sie Aufnahmen und Dateien mit einem anderen Rechner austauschen.")],
    defaultno => ($CONFIG{START_SAMBA} eq "yes") ? 0 : 1,
    height => 6);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{START_SAMBA} = "yes";
  } else {
    $CONFIG{START_SAMBA} = "no";
  }
}

sub FileSharingSelectWorkgroup {
  $return = $d->inputbox(
    text => gettext("Name der Arbeitsgruppe:"), 
    entry => SambaGetWorkgroup(),
    height => 8);
  if($d->state() eq "OK") {
    SambaSetWorkgroup($return);
  } else {
    Error();
  }
}

sub FileSharingChangePassword {
  do {
    $return = $d->password(
      text => gettext("Neues Passwort fr den Benutzer linvdr:"), 
      entry => "",
      height => 8);
    
    if($d->state() eq "OK") {
      if(length($return) >= 4) {
        my $retry = $d->password(
          text => gettext("Neues Passwort fr den Benutzer linvdr wiederholen:"), 
          entry => "",
          height => 8);
        if($d->state eq "OK") {
          if($return eq $retry) {
            SambaSetPassword("linvdr", $return);
            $d->msgbox(
              height => 5,
              width => 30,
              #text => ["[A=CENTER]Passwort gendert."],
              text => [gettext("    Passwort gendert.")],
            );
            return;
          } else {
            $d->msgbox(
              text => gettext("Passwrter stimmen nicht berein!"),
              height => 5,
            );
            $return = "";
          }
        } else {
          Error();
        }
      } else {
        $d->msgbox(
          text => gettext("Das Passwort darf nicht krzer als vier Zeichen sein!"),
          height => 5,
        );
      }
    } else {
      Error();
    }
  } while(length($return) < 4);
}

sub FileSharingStartFTPD {
  $return = $d->yesno(
    text => [gettext("Soll der FTP-Server gestartet werden? Damit knnen Sie Aufnahmen und Dateien mit einem anderen Rechner austauschen.")],
    defaultno => ($CONFIG{START_FTPD} eq "yes") ? 0 : 1,
    height => 6);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{START_FTPD} = "yes";
  } else {
    $CONFIG{START_FTPD} = "no";
  }
}

sub FileSharingStartDAAPD {
  $return = $d->yesno(
    text => [gettext("Soll der DAAP-Server gestartet werden? Damit knnen Sie die MP3s des Videorekorders anderen Rechnern/Benutzern im Netzwerk zugnglich machen.")],
    defaultno => ($CONFIG{START_DAAPD} eq "yes") ? 0 : 1,
    height => 6);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{START_DAAPD} = "yes";
  } else {
    $CONFIG{START_DAAPD} = "no";
  }
}

#############################################################################
# Plugins
#
sub PluginsMenu {
  my $VDRVersion = GetVDRVersion();

  if(!$VDRVersion) {
    $return = $d->msgbox(
      title => gettext("Fehler"),
      height => 12,
      text => [ 
        sprintf(gettext("\nKonnte VDR Version nicht identifizieren, Plugin Auswahl nicht mglich."))
      ]
    );
    exit;
  }
    

  my @plugins;
  for my $plugin (<$PLUGINDIR/libvdr-*.so.$VDRVersion>) {
    $plugin =~ /libvdr-(.*)\.so\..*$/;
    my $name = $1;
    next if($name eq "remote");
    next if($name eq "serial");
    next if($name eq "skincurses");
    next if($name eq "status");
    next if($name eq "osddemo");
    push(@plugins, $name);
    my $select;
    ($CONFIG{PLUGINLIST} =~ /$name/) ? ($select = 1) : ($select = 0);
    push(@plugins, [ GetPluginDescription($name), $select ]);
  }
  @return = $d->checklist(
    title => gettext("VDR-Erweiterungen fr VDR API-Version $VDRVersion"),
    text  => gettext("Plugins auswhlen:"),
    list  => [
      @plugins
    ],
    menuheight => 12
  );
  if($d->state() eq "OK") {
    if(scalar(@return)) {
      $CONFIG{PLUGINLIST} = join(" ", @return);
    } else {
      $CONFIG{PLUGINLIST} = "";
    }
  } else {
    return;
  }
}


#############################################################################
# LAN Networking
# 
sub NetworkLanMenu {
  my $exit = 0;
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("LAN-Einstellungen"),
      text  => "",
      menuheight => 9,
      height => 16,
      list  => [
        ($CONFIG{NETWORKING_LAN} ne "yes") ?
          (gettext("Netzwerk"), gettext("Netzwerk verwenden")) :
          (gettext("Netzwerk"), gettext("Kein Netzwerk verwenden"),
          gettext("Modul"), gettext("Netzwerktreiber auswhlen"),
          gettext("DHCP"), gettext("DHCP"),
          ($CONFIG{USE_DHCP_LAN} ne "yes") ? (
            gettext("IP"), gettext("IP ndern"),
            gettext("Gateway"), gettext("Gateway einstellen"),
            gettext("DNS"), gettext("DNS ndern"),
            gettext("Domain"), gettext("Domain"),
          ) : (),
          gettext("bersicht"), gettext("Alle Einstellungen auf einen Blick")),
        gettext("Zurck"), gettext("Zurck in das vorherige Men"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("Netzwerk")) {
      NetworkLanNetworking();
    }
    if($return eq gettext("Modul")) {
      NetworkLanAutodetectMenu();
    } 
    if($return eq gettext("IP")) {
      NetworkLanIPMenu();
    } 
    if($return eq gettext("Gateway")) {
      NetworkLanGatewayMenu();
    } 
    if($return eq gettext("DNS")) {
      NetworkLanDNSMenu();
    } 
    if($return eq gettext("Domain")) {
      NetworkLanDomainMenu();
    } 
    if($return eq gettext("DHCP")) {
      NetworkLanDHCPMenu();
    } 
    if($return eq gettext("bersicht")) {
      NetworkLanSummary();
    }
    if($return eq gettext("Zurck")) {
      $exit = 1;
    }
  }
}

sub NetworkLanNetworking {
  $return = $d->yesno(
    text => [gettext("Soll LinVDR in ein kabelgebundenes Netzwerk eingebunden werden?\n")],
    defaultno => ($CONFIG{NETWORKING_LAN} eq "yes") ? 0 : 1,
    height => 6);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{NETWORKING_LAN} = "yes";
  } else {
    $CONFIG{NETWORKING_LAN} = "no";
  }
}

sub NetworkLanSummary {
  if($CONFIG{USE_DHCP_LAN} ne "yes") {
    $return = $d->msgbox(
      title => gettext("Alle Einstellungen im berblick"),
      height => 12,
      text => [ 
        sprintf(gettext("\nNetzwerk-Modul: %s\nDHCP verwenden: %s\nIP: %s\nGateway: %s\nDNS: %s\nDomain: %s"),
        $CONFIG{NETWORK_DRIVER_LAN},
        ($CONFIG{USE_DHCP_LAN} eq "yes") ? gettext("ja") : gettext("nein"),
        $CONFIG{IP_LAN},
        $CONFIG{GATEWAY_LAN},
        $CONFIG{NAMESERVER_LAN},
        $CONFIG{DOMAIN_LAN})
      ]
    );
  } else {
    $return = $d->msgbox(
      title => gettext("Alle Einstellungen im berblick"),
      height => 7,
      text => [ 
        sprintf(gettext("\nNetzwerk-Modul: %s\nDHCP verwenden: %s"),
        $CONFIG{NETWORK_DRIVER_LAN},
        ($CONFIG{USE_DHCP_LAN} eq "yes") ? gettext("ja") : gettext("nein"))
      ]
    );
  }
}
  
sub NetworkLanIPMenu {
  $return = $d->inputbox(
    text => gettext("IP-Adresse von LinVDR:"), 
    entry => $CONFIG{IP_LAN},
    height => 8);
  if($d->state() eq "OK") {
    $CONFIG{IP_LAN} = $return;
  } else {
    Error();
  }
}

sub NetworkLanGatewayMenu {
  $return = $d->inputbox(
    text => gettext("IP-Adresse des Gateways:"), 
    entry => $CONFIG{GATEWAY_LAN},
    height => 8);
  if($d->state() eq "OK") {
    $CONFIG{GATEWAY_LAN} = $return;
  } else {
    Error();
  }
}

sub NetworkLanDomainMenu {
  $return = $d->inputbox(
    text => gettext("Domain des Netzwerks:"),
    entry => $CONFIG{DOMAIN_LAN},
    height => 8);
  if($d->state() eq "OK") {
    $CONFIG{DOMAIN_LAN} = $return;
  } else {
    Error();
  }
}

sub NetworkLanDNSMenu {
  $return = $d->inputbox(
    text => gettext("IP des Nameservers:"),
    entry => $CONFIG{NAMESERVER_LAN},
    height => 8);
  if($d->state() eq "OK") {
    $CONFIG{NAMESERVER_LAN} = $return;
  } else {
    Error();
  }
}

sub NetworkLanDHCPMenu {
  $return = $d->yesno(
    text => [gettext("Soll die Netzwerkkonfiguration ber DHCP erfolgen?\n(Erfordert einen entsprechenden Server im Netzwerk)")],
    defaultno => ($CONFIG{USE_DHCP_LAN} eq "yes") ? 0 : 1,
    height => 6);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{USE_DHCP_LAN} = "yes";
  } else {
    $CONFIG{USE_DHCP_LAN} = "no";
  }
}

sub NetworkLanAutodetectMenu {
  my($module, $vendorname, $devicename) = AutodetectNetworkDriver();
  $return = $d->yesno(
    title => gettext("Erkannte Netzwerkkarte"),
    text => [gettext("Hersteller:"), " $vendorname\n", gettext("Typ:"), " $devicename\n", gettext("Kernel-Modul:"), " $module\n \n", gettext("Mchten Sie diese Konfiguration verwenden?")],
    height => 9);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{NETWORK_DRIVER_LAN} = $module;
  } else {
    NetworkLanChooseNetworkModuleMenu();
  }
}

sub NetworkLanChooseNetworkModuleMenu {
  my @network_drivers;
  open(PIPE, "find $NETWORKDIR -name \"*.ko\"|");
  while(<PIPE>) {
    /.*\/(.*)\.ko/;
    my $select;
    ($CONFIG{NETWORK_DRIVER_LAN} eq $1) ? ($select = 1) : ($select = 0);
    push(@network_drivers, $1);
    push(@network_drivers, [ $1, $select ]);
  }
  close(PIPE);
  $return = $d->radiolist(
    title => gettext("Auswahl des erforderlichen Netzwerk-Moduls:"),
    list  => [ @network_drivers ],
    menuheight => 12
  );
  if($d->state() eq "OK") {
    $CONFIG{NETWORK_DRIVER_LAN} = $return;
  } else {
    Error(); 
  }
}

#############################################################################
# Other
#
sub OtherMenu() {
  my $exit = 0;
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("Sonstiges"),
      text  => "",
      menuheight => 6,
      height => 13,
      list  => [
        gettext("time"), gettext("Uhrzeit ber das Internet stellen"),
        gettext("weather"), gettext("Stations-IDs fr das Wetter-Plugin anzeigen"),
        gettext("hostname"), gettext("Rechnername ndern"),
        gettext("Zurck"), gettext("Zurck in das vorherige Men"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("time")) {
      OtherMenuSetTime();
    }
    if($return eq gettext("weather")) {
      OtherMenuWeatherCodes();
    }
    if($return eq gettext("hostname")) {
      OtherMenuChangeHostname();
    }
    if($return eq gettext("Zurck")) {
      $exit = 1;
    }
  }
}


sub OtherMenuWeatherCodes {
  my @codes = GetWeatherCodes();
  if(scalar(@codes)) {
    my $text;
    $d->menu(
      title => gettext("Stations-IDs fr Deutschland"),
      list  => [ @codes ],
      menuheight => 12,
    );
  } else {
    $d->msgbox(
      title => gettext("Fehler"),
      text  => gettext("Konnte keine Verbindung zu http://weather.noaa.gov aufbauen"),
      height=> 6
    );
  }
}

sub OtherMenuSetTime {
  if(!SetTime()) {
    $d->msgbox(
      title => gettext("Uhr stellen"),
      text => gettext("Uhr erfolgreich gestellt."),
      height => 6,
    );
  } else {
    $d->msgbox(
      title => gettext("Fehler"),
      text => gettext("Konnte Uhrzeit nicht stellen. Bitte beachten Sie, dass zum Stellen der Uhrzeit eine Internetverbindung bestehen muss."),
      height => 6,
    );
  }
}

sub OtherMenuChangeHostname {
  $return = $d->inputbox(
    text => gettext("Rechnername:"),
    entry => GetHostname(),
    height => 8);
  if($d->state() eq "OK") {
    SetHostname($return);
  } else {
    Error();
  }
}

#############################################################################
# Erweitertes Menu
sub AdvancedMenu() {
  my $exit = 0;
  while(!$exit) {
    my $return = $d->menu(
      title => gettext("Erweitert"),
      text  => "",
      menuheight => 9,
      height => 16,
      list  => [
        gettext("noflushd"), gettext("Festplatte bei inaktivitt abschalten"),
        gettext("ramdisk"), gettext("Gre der RAM-Disk verndern"),
        gettext("videotext"), gettext("Videotext-Daten auf RAM-Disk auslagern"),
        gettext("parameter"), gettext("Zustzlichen VDR-Parameter eingeben"),
        gettext("syslog"), gettext("Ringbuffer-Syslog ein/ausschalten"),
        gettext("hwdetect"), gettext("Hardware Erkennung"),
        gettext("vesamode"), gettext("VesaFB Auflsung ndern"),
        gettext("debtool"), gettext("Software Installieren"),
        gettext("newhd"), gettext("Weitere Festplatten einbinden"),
        gettext("Zurck"), gettext("Zurck in das vorherige Men"),
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq gettext("noflushd")) {
      AdvancedMenuNoflushd();
    } 
    if($return eq gettext("ramdisk")) {
      AdvancedMenuRamdiskSize();
    }
    if($return eq gettext("parameter")) {
      AdvancedMenuSetCommonParameter();
    }
    if($return eq gettext("videotext")) {
      AdvancedMenuShiftVTXData();
    }
    if($return eq gettext("syslog")) {
      AdvancedMenuSetRingbufferSyslog();
    }
    if($return eq gettext("hwdetect")) {
      AdvancedMenuHwDetect();
    }
    if($return eq gettext("vesamode")) {
      AdvancedVesaFBMode();
    }
    if($return eq gettext("debtool")) {
      AdvancedDEBTool();
    }
    if($return eq gettext("newhd")) {
      AdvancedMenuNewhd();
    }
    if($return eq gettext("Zurck")) {
      $exit = 1;
    }
  }
}

# XXX 
sub AdvancedMenuNewhd {
  my $exit = 0;
  my $ok = 0;
  my @disk = SearchHD();
  my $disk;
  my @hdlist;
  my $i = 0;
  my $hd;
  my $mntpt;
  my $partition;
  my $olddir;
  my $mntptfstabexist;
  my $mntptmtabexist;
  my $partnr = 1;

    foreach $disk(@disk){
      push(@hdlist, $disk);
      push(@hdlist, gettext("Formatieren und einbinden"));
      $i++;
    }
    push(@hdlist, gettext("Zurck"));
    push(@hdlist, gettext("Zurck in das vorherige Men"));

    while(!$exit) {
      my $return = $d->menu(
        title => gettext("Weitere Festplatten einbinden"),
        text  => gettext("Folgende Festplatten wurden gefunden:"),
        menuheight => $i + 1,
        height => $i + 8,
        list  => [
          @hdlist
        ]
      );
      ($exit = 1) if($d->state ne "OK");
      if($return eq gettext("Zurck")) {
        $exit = 1;
      }
      if($return =~ /hd[a-z]/) {
        $hd = $return;
        if ((!ReadTab($hd,"fstab")) && (!ReadTab($hd,"mtab"))) {
	  $return = $d->yesno (
	    title => gettext("ACHTUNG!!!"),
	    text => [gettext("Die Festplatte \"$hd\" wird formatiert.\nAlle Daten gehen dabei unwiderruflich verloren.\n\nTrotzdem fortfahren?")],
	    height => 8,
	    defaultno => 1,
	  );
          if($return) {
	    $ok = 0;
	    while(!$ok) {
	      $return = $d->menu (
	        text => gettext("Mountpoint auswhlen"),
	        menuheight => 11,
	        height => 18,
		list => [
		gettext ("/pub"), gettext("Festplatte als /pub einbinden"),
		gettext ("/pub/export"), gettext("Festplatte als /pub/export einbinden"),
		gettext ("/pub/images"), gettext("Festplatte als /pub/images einbinden"),
		gettext ("/pub/mp3"), gettext("Festplatte als /pub/mp3 einbinden"),
		gettext ("/pub/mplayer"), gettext("Festplatte als /pub/mplayer einbinden"),
		gettext ("/video0"), gettext("Festplatte als /video0 einbinden"),
		gettext ("/video1"), gettext("Festplatte als /video1 einbinden"),
		gettext ("/video2"), gettext("Festplatte als /video2 einbinden"),
		gettext ("/video3"), gettext("Festplatte als /video3 einbinden"),
		gettext ("Zurck"), gettext ("Zurck in das vorherige Men"),
		]
	      );
	      if (($return eq "Zurck") | ($d->state ne "OK")) {
	        $ok = 1;
	      }
	      else {
	        $mntpt = $return;
		$mntptfstabexist = 0;
		$mntptmtabexist = 0;
		foreach $partition(ReadTab("hd[a-z]", "fstab")) {
		  if ($partition->{mountpoint} eq $mntpt) {
		    $mntptfstabexist = 1;
		    last;
		  }
		}
		if ($mntptfstabexist == 1) {
		    $return = $d->yesno (
		      title => gettext("Mountpoint ndern?"),
		      text => [gettext("Die Partition $partition->{partition} ist bereits als $mntpt eingebunden.\nMchten Sie den Mountpoint ndern?\n\nHinweis:\nWenn Sie \"No\" whlen, wird die neue Festplatte als $mntpt\neingebunden und das existierende Verzeichnis auf die neue Festplatte verschoben.\nStellen Sie sicher, dass die neue Festplatte ausreichend gross ist.")],
		      height => 13,
		      defaultno => 0,
		    );
		    if(!$return){
		      $olddir = $mntpt.".old";
		      system ("/etc/init.d/runvdr stop >/dev/null");
		      FormatDisk($hd);
		      system ("umount $mntpt");
		      system ("mkdir -p $olddir");
		      system ("mount -t ext3 $partition->{partition} $olddir");
		      NewFstab(("/dev/$hd$partnr"), $mntpt);
		      system("mount $mntpt");
		      system("cp -a $olddir/* $mntpt");
		      system ("umount $olddir");
		      system("rm -fr $olddir");
		      system ("/etc/init.d/runvdr start");
		      $ok = 1;
		    }
	        }
		else {
		  foreach $partition(ReadTab("hd[a-z]","mtab")) {
		    if ($partition->{mountpoint} eq $mntpt) {
		      $mntptmtabexist = 1;
		      last;
		    }
		  }
		  if ($mntptmtabexist == 1) {
		    $return = $d->menu (
		      text => [gettext ("Die Partition $partition->{partition} ist momentan als $mntpt gemountet.\nBitte whlen Sie, wie fortgefahren werden soll.")],
		      menuheight => 4,
		      height => 11,
		      list => [
		        gettext ("Kopieren"), gettext ("Gemountete Partition wird auf die neue Festplatte kopiert"),
			gettext ("Unmounten"), gettext ("Partition unmounten und neue Festplatte als $mntpt einbinden"),
			gettext ("Zurck"), gettext ("Anderen Mountpoint auswhlen"),
		      ]
		    );
		    if ($return eq gettext ("Kopieren")) {
		      $olddir = $mntpt.".old";
		      system ("/etc/init.d/runvdr stop >/dev/null");
		      FormatDisk ($hd);
		      system ("umount $mntpt");
		      system ("mkdir -p $olddir");
		      system ("mount -t ext3 $partition->{partition} $olddir");
		      NewFstab (("/dev/$hd$partnr"), $mntpt);
		      system ("mount $mntpt");
		      system ("cp -a $olddir/* $mntpt");
		      system ("umount $olddir");
		      system ("rm -fr $olddir");
		      system ("/etc/init.d/runvdr start");
		      $ok = 1;
		    }
		    if ($return eq gettext ("Unmounten")) {
		      system ("/etc/init.d/runvdr stop >/dev/null");
		      FormatDisk ($hd);
		      system ("umount $mntpt");
		      NewFstab (("/dev/$hd$partnr"), $mntpt);
		      system ("mount $mntpt");
		      system ("/etc/init.d/runvdr start");
		      $ok = 1;
		    }
		  }
		  else {
		    if (-d $mntpt){
		      $return = $d->yesno (
		        title => gettext("Mountpoint ndern?"),
		        text => [gettext("Das Verzeichnis $mntpt existiert bereits.\nMchten Sie den Mountpoint ndern?\n\nHinweis:\nWenn Sie \"No\" whlen, wird die neue Festplatte als $mntpt\neingebunden und das existierende Verzeichnis auf die neue Festplatte verschoben.\nStellen Sie sicher, dass die neue Festplatte ausreichend gross ist.")],
		        height => 13,
		        defaultno => 0,
		      );
		      if (!$return) {
		        $olddir = $mntpt.".old";
		        system ("/etc/init.d/runvdr stop >/dev/null");
			FormatDisk($hd);
			system ("mv $mntpt $olddir");
			system ("mkdir -p $mntpt");
			NewFstab (("/dev/$hd$partnr"), $mntpt);
			system ("mount $mntpt");
			system ("cp -a $olddir/* $mntpt");
			system ("rm -fr $olddir");
			system ("/etc/init.d/runvdr start");
			$ok = 1;
		      }
		    }
		    else {
		      FormatDisk($hd);
		      system ("mkdir $mntpt >/dev/null 2>/dev/null");
		      NewFstab (("/dev/$hd$partnr"), $mntpt);
		      system ("mount $mntpt");
		      $ok = 1;
		    }
		  }
		}
	      }
	    }
	  }
	}
	else {
	  $d->msgbox (
	    text => [gettext("NEIN! NJET! NO!\nDiese Funktion ist dafr gedacht auf einer NEUEN Festplatte\nEINE Partition anzulegen und zu formatiert.\nDie Festplatte $hd ist bereits in das System eingebunden.")],
	    height => 8,
	    width => 65,
	  );
        }
      }
    }
}
sub AdvancedMenuHwDetect {
  my $usbhcdmodules = join(" ", AutodetectUSBHCDriver());
  my $usbmodules = join(" ", AutodetectUSBModules());
  my $soundmodules = join(" ", AutodetectSoundDriver());
  my $cdrom = join(" ", SearchCDROM());
  my($networkmodule, undef, undef) = AutodetectNetworkDriver();
  
  if(!length($usbhcdmodules)) {
    $usbhcdmodules = gettext("nicht gefunden");
  }
  if(!length($usbmodules)) {
    $usbmodules = gettext("nicht gefunden");
  }
  if(!length($soundmodules)) {
    $soundmodules = gettext("nicht gefunden");
  }
  if(!length($cdrom)) {
    $cdrom = gettext("nicht gefunden");
  }
  if(!length($networkmodule)) {
    $networkmodule = gettext("nicht gefunden");
  }
  $return = $d->yesno(
    title => gettext("Hardware Erkennung"),
    text => [gettext("Erkannte Hardware:\n\nUSB Host Controller: $usbhcdmodules\nUSB Geraete: $usbmodules\nSoundkarte: $soundmodules\nCDROM: $cdrom\nNetzwerkkarte: $networkmodule\n\nEinstellungen uebernehmen?")],
    height => 14,
    defaultno => 0,
    );
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{NETWORK_DRIVER_LAN} = $networkmodule;
    open(FILE, ">$MODULES_SETUP");
    my $text = sprintf("#\n# generated by SETUP at %s\n# DO NOT EDIT THIS FILE\n#\n", 
      strftime("%d.%m.%Y %H:%M:%S", localtime(time())));
    print FILE $text;

    if($return) {
      if($usbhcdmodules ne gettext("nicht gefunden")) {
        print FILE join("\n", split(" ", $usbhcdmodules));
        print FILE "\n";
      }
      if($usbmodules ne gettext("nicht gefunden")) {
        print FILE join("\n", split(" " , $usbmodules));
        print FILE "\n";
      }
      if($soundmodules ne gettext("nicht gefunden")) {
        print FILE join("\n", split(" " , $soundmodules));
        print FILE "\n";
      }
    }
    close(FILE);
  }
}

sub AdvancedDEBTool {
  my %PACKAGES = (
    "Paketliste Updaten", "debtool -u",
    "Midnight Commander", "debtool -i libgpmg1 mc mc-common file libglib1.2",
    "gcc",                "debtool -i gcc-2.95 make libc6-dev cpp-2.95 libjpeg62-dev binutils libstdc++2.10-dev g++-2.95; ln -s /usr/bin/gcc-2.95 /usr/bin/gcc; ln -s /usr/bin/gcc /usr/bin/cc; ln -s /usr/bin/g++-2.95 /usr/bin/g++;",
    "vim",                "debtool -i vim libgpmg1",
  );

  my @packages;
  for my $package (keys(%PACKAGES)) {
    push(@packages, $package);
    push(@packages, "");
  }
  my $return = $d->menu(
    text  => gettext("Software Installieren:"),
    menuheight => 5,
    height => 12,
    list  => [ @packages ],
  );
  
  if($return) {
    system($PACKAGES{"$return"});
    sleep(5);
  }
}

sub AdvancedVesaFBMode {
  my $param = GetVesaFBMode();
 
  my(@sizes, $select);

  my %VESAFBMODES = (
    "0x000" => "aus",
    "0x311" => "640x480",
    "0x314" => "800x600",
    "0x317" => "1024x768",
    "0x31a" => "1280x1024",
  );

  for my $mode (keys(%VESAFBMODES)) {
    if($mode eq $param) {
      $select = 1;
    } else {
      $select = 0;
    }
      
    push(@sizes, $mode);
    push(@sizes, [ $VESAFBMODES{$mode}, $select ]);
  }
  my $return = $d->radiolist(
    text  => gettext("VesaFB-Modus:"),
    menuheight => 5,
    height => 12,
    list  => [ @sizes ],
  );
  
  if($return) {
    SetVesaFBMode($return);
  }
}

sub AdvancedMenuSetRingbufferSyslog {
  $return = $d->yesno(
    title => gettext("Ringbuffer-Syslog ein/ausschalten?"),
    text => [gettext("Soll der Ringbuffer ausgeschaltet werden?\n\nMeldungen des Syslogs werden dann in /var/log/messages gespeichert. Hinweis: Es muss darauf geachtet werden, dass die Datei nicht grer als die RAM-Disk wird!")],
    height => 9,
    defaultno => 1,
    );
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{RINGBUFFER_SYSLOG} = "no";
  } else {
    $CONFIG{RINGBUFFER_SYSLOG} = "yes";
  }
}

sub AdvancedMenuShiftVTXData {
  $return = $d->yesno(
    title => gettext("Videotext-Daten auf RAM-Disk auslagern"),
    text => [gettext("RAM-Disk wird vergrert, /vtx gelscht und nach /ramdisk verlinkt.")],
    height => 9);
  if($d->state() eq "ESC") {
    Error();
  }
  my $vtx_data = 128; # ramdisk groesse
  my $free = 64; # muss freibleiben
  if($return) {
    my $system_memory = GetSystemMemory();
    if(($system_memory - $vtx_data) < $free) {
      $return = $d->msgbox(
        text => [gettext("Zuwenig Hauptspeicher!")],
        height => 6,
        width => 40);
      return;
    }
    $CONFIG{RAMDISK_SIZE} = $vtx_data;
    system("/etc/init.d/runvdr stop");
    system("rm -fr /vtx");
    system("mkdir -p /ramdisk/vtx");
    system("ln -s /ramdisk/vtx /vtx");
    system("/etc/init.d/runvdr start");
  } 
}

sub AdvancedMenuSetCommonParameter {
  $return = $d->inputbox(
    text => [gettext("Zustzlichen VDR-Parameter eingeben:")],
    entry => $CONFIG{COMMON_PARAMETER},
    height => 8);
  if($d->state() eq "OK") {
    $CONFIG{COMMON_PARAMETER} = $return;
  } 
}

sub AdvancedMenuRamdiskSize {
  my $ramdisk_size = $CONFIG{RAMDISK_SIZE};
 
  if($ramdisk_size == 0) {
    $ramdisk_size = 4;
  }

  my(@sizes, $select);
  for(my $size = 4; $size <= 16; $size += 4) {
    if($ramdisk_size == $size) {
      $select = 1;
    } else {
      $select = 0;
    }
      
    push(@sizes, $size);
    push(@sizes, [ "${size}MB", $select ]);
  }
  my $return = $d->radiolist(
    text  => gettext("Gre der RAM-Disk:"),
    menuheight => 5,
    height => 12,
    list  => [ @sizes ],
  );
  
  if($return) {
    $CONFIG{RAMDISK_SIZE} = $return;
  }
}

sub AdvancedMenuNoflushd {
  $return = $d->yesno(
    text => [gettext("Soll die Festplatte bei inaktivitt abgeschaltet werden? (Experimentiell)")],
    defaultno => ($CONFIG{START_NOFLUSHD} eq "yes") ? 0 : 1,
    height => 7);
  if($d->state() eq "ESC") {
    Error();
  }
  if($return) {
    $CONFIG{START_NOFLUSHD} = "yes";
    $CONFIG{START_SAMBA} = "no";
    $CONFIG{RINGBUFFER_SYSLOG} = "yes";
    $CONFIG{START_CRON} = "no";
    $return = $d->inputbox(
      text => [gettext("Zeit bis zum Abschalten der Festplatten (Minuten):")],
      entry => $CONFIG{NOFLUSHD_TIMEOUT},
      height => 8);
    if($d->state() eq "OK") {
      $CONFIG{NOFLUSHD_TIMEOUT} = $return;
      $return = $d->inputbox(
        text => [gettext("Abzuschaltende Festplatte(n) (z.B. /dev/hda):")],
        entry => $CONFIG{NOFLUSHD_DISKS},
        height => 8);
      if($d->state() eq "OK") {
        $CONFIG{NOFLUSHD_DISKS} = $return;
      } else {
        Error();
      }
    } else {
      Error();
    }
  } else {
    $CONFIG{START_NOFLUSHD} = "no";
  }
}


#############################################################################
#
#
#
#
#
#############################################################################
#
sub GetSystemMemory {
  my $memory;
  open(FILE, "/proc/meminfo") || die "cant read /proc/meminfo ($!)";
  while(<FILE>) {
    if(/MemTotal:\s* (\d+) kB/) {
     $memory = $1;
    }
  }
  close(FILE);
  return(int($memory / 1024));
}

sub GetVDRVersion {
  my $vdr = "/usr/bin/vdr";
  my $symlink = readlink($vdr);
  my $cmd = join "", $symlink, " -V";
  my $retval = qx($cmd);
  if($retval =~ /.*[\(\/](.*)[\/\)]/) {
    return $1;
  } else {
    return (false);
  }
}

sub GetHostname {
  open(FILE, "/etc/hostname");
  my $value = join("", <FILE>);
  chomp($value);
  close(FILE);
  return($value);
}

sub SetHostname {
  my $hostname = shift;
  open(FILE, ">/etc/hostname");
  print FILE "$hostname\n";
  close(FILE);
}

sub GetLanguage {
  open(FILE, "/etc/language");
  my $value = join("", <FILE>);
  chomp($value);
  close(FILE);
  return($value);
}

sub GetChannelConfigs {
  my $CHANNELS_DIR = "/etc/vdr/channels";
  my @list;
  while(<$CHANNELS_DIR/DVB-*>) {
    /^$CHANNELS_DIR\/(DVB-[sct])_(.*).conf/;
    my $media = $1;
    my $host = $2;
    $host =~ s/_/ /;
    $_ =~ /.*\/(.*\.conf)$/;
    push(@list, {
      media    => $media,
      host     => $host,
      file     => $1,
      filename => $_,
    });
  }
  return(@list);
}

sub GetVesaFBMode {
  open(FILE, $GRUB_CONFIG) || die "cant open $GRUB_CONFIG";
  while(<FILE>) {
    # kernel    /boot/vmlinuz vga=0x314 root=/dev/hda1 ramdisk_size=4096 quiet
    if(/^kernel[ \t]+\/boot\/vmlinuz vga=(0x...) /) {
      return($1);
    }
  }
  close(FILE);
  return(0);
}

sub SetVesaFBMode {
  my $param = shift;
  
  return if(!$param);
  my @file;
  open(FILE, $GRUB_CONFIG) || die "cant open $GRUB_CONFIG";
  while(<FILE>) {
    if(/^kernel[ \t]+\/boot\/vmlinuz vga=(0x...) /) {
      $_ =~ s/vga=$1/vga=$param/;
    }
    push(@file, $_);
  }
  close(FILE);

  open(FILE, ">$GRUB_CONFIG") || die "cant open $GRUB_CONFIG";
  print FILE @file;
  close(FILE);
}

#XXX OBSOLETE
sub GetRamdiskSize {
  open(FILE, $GRUB_CONFIG) || die "cant open $GRUB_CONFIG";
  while(<FILE>) {
    if(/^kernel[ \t]+\/boot\/vmlinuz .* ramdisk_size=(\d+)/) {
      return($1);
    }
  }
  close(FILE);
  return(0);
}

#XXX OBSOLETE
sub SetRamdiskSize {
  my $size = shift;
  
  return if(!$size);
  my @file;
  open(FILE, $GRUB_CONFIG) || die "cant open $GRUB_CONFIG";
  while(<FILE>) {
    if(/^kernel[ \t]+\/boot\/vmlinuz .* ramdisk_size=(\d+)/) {
      $_ =~ s/ramdisk_size=$1/ramdisk_size=$size/;
    }
    push(@file, $_);
  }
  close(FILE);

  open(FILE, ">$GRUB_CONFIG") || die "cant open $GRUB_CONFIG";
  print FILE @file;
  close(FILE);
}

sub SetTime {
  system("rdate ptbtime2.ptb.de");
  return($?);
}

sub GetWeatherCodes {
  my $ua = LWP::UserAgent->new;
  my $req = HTTP::Request->new(GET => "http://weather.noaa.gov/weather/DE_cc.html");
  my $res = $ua->request($req);
  
  my @codes;
  if($res->is_success) {
    for(split("\n", $res->content)) {
      if(/<option value="(\w\w\w\w)"> (\w+)/i) {
        push(@codes, $1);
        push(@codes, $2);
      }
    }
    return(@codes);
  } else {
    print gettext("Fehlgeschlagen: Ist vielleicht die Netzwerkverbindung fehlerhaft?\n");
  }
}

sub ReadDisclaimer {
  my $content;
  if(-e $DISCLAIMER) {
    open(FILE, $DISCLAIMER);
    $content = join("", <FILE>);
    close(FILE);
    return($content);
  } else {
    return(gettext("Die Datei $DISCLAIMER fehlt."));
  }
}

sub SystemSetPassword {
  my $password = shift;
  return if(!$password);
  my @content;
  open(FILE, "/etc/shadow");
  while(<FILE>) {
    my @line = split(":");
    if($line[0] eq "root") {
      $line[1] = crypt($password, substr($password, 0, 1) . substr($password, length($password) - 1, 1));
      push(@content, join(":", @line));
    } else {
      push(@content, $_);
    }
  }
  close(FILE);
  open(FILE, ">/etc/shadow");
  print FILE @content;
  close(FILE);
}

sub SambaSetPassword {
  my($user, $pass) = @_;
  system("smbpasswd $user $pass");
}

sub SambaGetWorkgroup {
  open(FILE, $SMBCONF);
  while(<FILE>) {
    if(/^([ \t]*workgroup[ \t]*)\=[ \t]*([\w\d ]+)/) {
      close(FILE);
      return($2);
    }
  }
  close(FILE);
}

sub SambaSetWorkgroup {
  my $workgroup = shift;
  my @content;
  open(FILE, $SMBCONF);
  while(<FILE>) {
    if(/^([ \t]*workgroup[ \t]*)\=[ \t]*([\w\d ]+)/) {
      push(@content, "$1= $workgroup\n");
    } else {
      push(@content, $_);
    }
  }
  close(FILE);
  open(FILE, ">$SMBCONF");
  print FILE @content;
  close(FILE);
}

sub SearchHD {
  my @disk;
  for my $item (</proc/ide/hd[a-z]>) {
    open(FILE, "$item/media");
    my $media = <FILE>;
    chomp($media);
    close(FILE);
    $item =~ /\/(hd[a-z])$/;
    my $device = $1;
    if($media eq "disk") {
      push(@disk, $device);
    }
  }
  return(@disk);
}

sub SearchCDROM {
  my @cdrom;
  for my $item (</proc/ide/hd[a-z]>) {
    open(FILE, "$item/media");
    my $media = <FILE>;
    chomp($media);
    close(FILE);
    $item =~ /\/(hd[a-z])$/;
    my $device = $1;
    if($media eq "cdrom") {
      push(@cdrom, $device);
    }
  }
  return(@cdrom);
}

sub GetPluginDescription {
  my $plugin = shift;
  return $PLUGINS->{$plugin} ? $PLUGINS->{$plugin} : gettext("Keine Beschreibung vorhanden");
}
  
sub WriteConfig {
  my $text = sprintf("#\n# generated by SETUP at %s\n#\n", 
    strftime("%d.%m.%Y %H:%M:%S", localtime(time())));
  for my $key (sort(keys(%CONFIG))) {
    if($key) {
      $text .= "$key=\"" . $CONFIG{$key} . "\"\n";
    }
  }
  open(FILE, ">$SYSCONFIG");
  print FILE $text;
  close(FILE);
}

sub ReadConfig {
  if(-e $SYSCONFIG) {
    open(FILE, $SYSCONFIG);
    while(<FILE>) {
      next if(/^#/);   
      /(.*)="(.*)"/;
      my($key, $value) = ($1, $2);
      $CONFIG{$key} = $value;
    }
    close(FILE);
    return(0);
  } else {
    return(1);
  }
}

sub Error {
  $return = $d->msgbox(
    text => [gettext("Konfiguration abgebrochen!")],
    height => 6,
    width => 40);
  exit(1);
}

sub AutodetectUSBModules {
  my @array;
  if(-e "/proc/bus/usb") {
    for my $dir (</proc/bus/usb/00?/*>) {
      my $info = `usbmodules --device $dir`;
      chomp($info);
      if(length($info)) {
        push(@array, $info);
      }
    }
  }
  return(@array);
}

sub AutodetectNetworkDriver {
  my @Dev;
  ReadDriverMap(\@Dev, $PCIDB, $PCIMAP);

  foreach my $dev (GetPciInfo()) {
    # Nur Netzwerkkarten (PCI-Class 0x0200) ausgeben
    # Siehe http://www.pcidatabase.com/pci_c_header.php unter "PciClassCodeTable"
    if ($dev->{"class"} == 0x0200) {
    #  printf("%4x\t%4x:%4x\t%12s\t%s\n", 
    #    $dev->{"class"}, $dev->{"vendor"}, $dev->{"device"},
    #    $Dev[$dev->{"vendor"}][$dev->{"device"}]{"module"},
    #    $Dev[$dev->{"vendor"}][$dev->{"device"}]{"name"});
      return($Dev[$dev->{"vendor"}][$dev->{"device"}]{"module"}, $Dev[$dev->{"vendor"}][$dev->{"device"}]{"vendor"}, $Dev[$dev->{"vendor"}][$dev->{"device"}]{"device"});
    }
  }
}

sub AutodetectSoundDriver {
  my @Dev;
  ReadDriverMap(\@Dev, $PCIDB, $PCIMAP);

  foreach my $dev (GetPciInfo()) {
    # Nur Soundkarten (PCI-Class 0x0401) ausgeben
    # Siehe http://www.pcidatabase.com/pci_c_header.php unter "PciClassCodeTable"
    if ($dev->{"class"} == 0x0401) {
    #  printf("%4x\t%4x:%4x\t%12s\t%s\n", 
    #    $dev->{"class"}, $dev->{"vendor"}, $dev->{"device"},
    #    $Dev[$dev->{"vendor"}][$dev->{"device"}]{"module"},
    #    $Dev[$dev->{"vendor"}][$dev->{"device"}]{"name"});
    #  return(
    #    $Dev[$dev->{"vendor"}][$dev->{"device"}]{"module"}, 
    #    $Dev[$dev->{"vendor"}][$dev->{"device"}]{"vendor"}, 
    #    $Dev[$dev->{"vendor"}][$dev->{"device"}]{"device"});
      if(length($Dev[$dev->{"vendor"}][$dev->{"device"}]{"module"}) > 0) {
        return($Dev[$dev->{"vendor"}][$dev->{"device"}]{"module"});
      } else {
        return(());
      }
    }
  }
}

sub AutodetectUSBHCDriver {
  my %Drivers;
  
  foreach my $dev (GetPciInfo()) {
    # USB-Controller (PCI-Class 0x0300) ausgeben
    # Siehe http://www.pcidatabase.com/pci_c_header.php unter "PciClassCodeTable"
    if ($dev->{"class"} == 0x0300) {
      $Drivers{"uhci-hcd"} = 1;
    } elsif ($dev->{"class"} == 0x0310) {
      $Drivers{"ohci-hcd"} = 1;
    } elsif ($dev->{"class"} == 0x0320) {
      $Drivers{"ehci-hcd"} = 1;
    }
  }
  return(keys(%Drivers));
}

#
# Zuordnung PCI-IDs <-> Kernel-Modul aus /lib/modules/*/modules.pcimap einlesen
# und Hersteller- und Gertenamen aus /etc/pci.db bestimmen
sub ReadDriverMap {
  my($vendor, $device, $module, $vendorname, $devicename);
  my $pcidb;
  my $modline;
  my $Pci = $_[0];

  open(PciDB, $_[1]);
  $pcidb = join("", <PciDB>);
  close(PciDB);  

  open(PciMap, $_[2]);
  while (! eof(PciMap)) {
    $modline = <PciMap>;
    # Modulname, Vendor-ID und Device-ID herauslsen
    $modline =~ /^([^#][\w-]*)\s+0x0000([0-9A-Fa-f]+)\s+0x0000([0-9A-Fa-f]+)\s+/;
    if(length($1) > 0) {
      ($vendor, $device, $module) = ($2, $3, $1);
      $$Pci[hex("0x".$vendor)][hex("0x".$device)]{"module"} = $module;
      # Herstellernamen bestimmen
      $pcidb =~ /^v\t$vendor\t([^\t]*)/m;
      $$Pci[hex("0x".$vendor)][hex("0x".$device)]{"vendor"} = $1;
      # Gertenamen bestimmen
      $pcidb =~ /^d\t$vendor$device\t([^\t]*)/m;
      $$Pci[hex("0x".$vendor)][hex("0x".$device)]{"device"} = $1;
    }
  }
  close(PciMap);

  return $Pci;
}

# PCI-Class, -Vendor-ID und -Device-ID direkt aus /proc/bus/pci auslesen
sub GetPciInfo {
  my $dev;
  my @Pci;
  
  foreach $dev (</proc/bus/pci/??/*>) {
    my $buffer;
    my($class, $vendor, $device);
    
    open(Dev, $dev);
    # Vendor- und Device-ID stehen gleich am Anfang, Class ist Byte 11 und 12
    read(Dev, $buffer, 12, 0);
    ($vendor, $device, $class) = unpack("SSS", substr($buffer,0,4) . substr($buffer,-2,2));
    close(Dev);
    push(@Pci, {"class" => $class, "vendor" => $vendor, "device" => $device});
  }
  return @Pci;
}

sub ReadTab {
  my $device = shift;
  my $file = shift;
  my @list;
  my $fstabmntpt;

  open(FILE, "/etc/$file");
  while(<FILE>) {
    if(/^(\/dev\/($device)[\d+])[ \t]+(.+?)[ \t]+(.+?)[ \t]/) {
      if ($4 eq "swap"){
        $fstabmntpt = $4;
      }
      else {
        $fstabmntpt = $3;
      }
      push (@list, {
        device     => $2,
        partition  => $1,
        mountpoint => $fstabmntpt,
      });
    }
  }
  close(FILE);
  return(@list);
}

sub NewFstab {
  my $partition = shift;
  my $mntpt = shift;
  my @content;
    open(FILE, "/etc/fstab");
    while(<FILE>) {
      next if(/^(.*?)[\d+][ \t]+($mntpt)[ \t]+(.+?)[ \t]/);
      push(@content, $_);
    }
    push(@content, "$partition	$mntpt	ext3	defaults,errors=remount-ro	0 0\n");
    close(FILE);
    open(FILE, ">/etc/fstab");
    print FILE @content;
    close(FILE);
}

sub FormatDisk {
  my $hd = shift;
  my $partition = $hd."1";
  system("dd if=/dev/zero of=/dev/$hd bs=1024 count=2 >/dev/null 2>/dev/null");
  system("fdisk /dev/$hd >/dev/null 2>/dev/null << EOF
n
p
1


w
EOF");
  system("mke2fs -q -j -o Linux -L $hd /dev/$partition >/dev/null 2>/dev/null");
}

